// Must link against the following libs: shlwapi.lib Propsys.lib shell32.lib
#include <Windows.h>

#include <ShObjIdl.h>
#include <propsys.h>
#include <propkey.h>
#include <propvarutil.h>

#include <iostream>

using namespace std;

HWND GetAppWindow() { return GetConsoleWindow(); }

void GetAppUserModelProperty(const PROPERTYKEY &key, LPWSTR value, size_t valueSize)
{
	HWND hwnd = GetAppWindow();
	if (hwnd != NULL)
	{
		IPropertyStore *store = NULL;
		HRESULT hr = SHGetPropertyStoreForWindow(hwnd, IID_PPV_ARGS(&store));
		if (SUCCEEDED(hr))
		{
			PROPVARIANT propVariant;
			hr = store->GetValue(key, &propVariant);
			if (SUCCEEDED(hr))
			{
				hr = PropVariantToString(propVariant, value, valueSize);
				if (FAILED(hr))
				{
					wcerr << L"PropVariantToString failure: 0x" << hex << hr << endl;
				}
			}
			else
			{
				wcerr << L"GetValue failure: 0x" << hex << hr << endl;
			}

			store->Release();
			store = NULL;
		}
		else
		{
			wcerr << L"SHGetPropertyStoreForWindow failure: 0x" << hex << hr << endl;
		}
	}
	else
	{
		wcerr << L"No foreground window." << endl;
	}
}

void SetAppUserModelProperty(const PROPERTYKEY &key, LPCWSTR value)
{
	HWND hwnd = GetAppWindow();
	if (hwnd != NULL)
	{
		IPropertyStore *store = NULL;
		HRESULT hr = SHGetPropertyStoreForWindow(hwnd, IID_PPV_ARGS(&store));
		if (SUCCEEDED(hr))
		{
			PROPVARIANT propVariant;
			hr = InitPropVariantFromString(value, &propVariant);
			if (SUCCEEDED(hr))
			{
				hr = store->SetValue(key, propVariant);
				if (SUCCEEDED(hr))
				{
					hr = store->Commit();
					if (FAILED(hr))
					{
						wcerr << L"Commit failure: 0x" << hex << hr << endl;
					}
				}
				else
				{
					wcerr << L"SetValue failure: 0x" << hex << hr << endl;
				}
			}
			else
			{
				wcerr << L"InitPropVariantFromString failure: 0x" << hex << hr << endl;
			}

			store->Release();
			store = NULL;
		}
		else
		{
			wcerr << L"SHGetPropertyStoreForWindow failure: 0x" << hex << hr << endl;
		}
	}
	else
	{
		wcerr << L"No foreground window." << endl;
	}
}

struct AppUserModelProperties
{
	LPCWSTR id;
	LPCWSTR cmd;
	LPCWSTR name;
	LPCWSTR icon;
};

void SetAppUserModelProperties(const AppUserModelProperties &props)
{
	if (props.id)
	{
		SetAppUserModelProperty(PKEY_AppUserModel_ID, props.id);
	}
	if (props.cmd)
	{
		SetAppUserModelProperty(PKEY_AppUserModel_RelaunchCommand, props.cmd);
	}
	if (props.name)
	{
		SetAppUserModelProperty(PKEY_AppUserModel_RelaunchDisplayNameResource, props.name);
	}
	if (props.icon)
	{
		SetAppUserModelProperty(PKEY_AppUserModel_RelaunchIconResource, props.icon);
	}
}

void GetAppUserModelProperties()
{
	WCHAR buffer[1024] = L"";
	GetAppUserModelProperty(PKEY_AppUserModel_ID, buffer, ARRAYSIZE(buffer));
	wcout << L"AppUserModel ID: " << buffer << endl;

	GetAppUserModelProperty(PKEY_AppUserModel_RelaunchCommand, buffer, ARRAYSIZE(buffer));
	wcout << L"AppUserModel RelaunchCommand: " << buffer << endl;

	GetAppUserModelProperty(PKEY_AppUserModel_RelaunchDisplayNameResource, buffer, ARRAYSIZE(buffer));
	wcout << L"AppUserModel RelaunchDisplayNameResource: " << buffer << endl;

	GetAppUserModelProperty(PKEY_AppUserModel_RelaunchIconResource, buffer, ARRAYSIZE(buffer));
	wcout << L"AppUserModel RelaunchIconResource: " << buffer << endl;
}

bool MatchCommand(LPCWSTR command, LPCWSTR test)
{
	if (test[0] == L'/')
	{
		test++;
	}
	return 0 == _wcsicmp(command, test);
}

int wmain(int argCount, wchar_t *args[])
{
	bool showUsage = false;

	if (argCount > 1)
	{
		AppUserModelProperties props = { NULL };

		if (argCount == 2)
		{
			props.id = args[1];
		}
		else
		{
			for (int argIdx = 1; !showUsage && argIdx < argCount; ++argIdx)
			{
				if (MatchCommand(L"id", args[argIdx]) && argIdx + 1 < argCount)
				{
					props.id = args[++argIdx];
				}
				else if (MatchCommand(L"cmd", args[argIdx]) && argIdx + 1 < argCount)
				{
					props.cmd = args[++argIdx];
				}
				else if (MatchCommand(L"name", args[argIdx]) && argIdx + 1 < argCount)
				{
					props.name = args[++argIdx];
				}
				else if (MatchCommand(L"icon", args[argIdx]) && argIdx + 1 < argCount)
				{
					props.icon = args[++argIdx];
				}
				else
				{
					wcerr << L"Unknown parameter or missing value: " << args[argIdx] << endl;
					showUsage = true;
				}
			}
		}

		SetAppUserModelProperties(props);
		GetAppUserModelProperties();
	}
	else
	{
		wcout << L"Current window's AppUserModel properties:" << endl;
		GetAppUserModelProperties();
		wcout << endl;
		showUsage = true;
	}

	if (showUsage)
	{
		wcout << L"Usage:" << endl
			<< L"\tSetAppUserModelId [AppUserModelId]" << endl
			<< L"\tSetAppUserModelId (/id [AppUserModelId]) (/cmd [RelaunchCommand]) (/name [RelaunchDisplayNameResource]) (/icon [RelaunchIconResource])" << endl
			<< endl
			<< L"AppUserModelId may not be longer than 128 characters or contain spaces and must be of the syntax [CompanyName].[ProductName].[SubProduct].[VersionInformation]" << endl
			<< L"RelaunchCommand is a string that will be executed via ShellExecute. Note that this means the command could be a URL." << endl
			<< L"RelaunchDisplayNameResource may either be a literal string or a resource string prefixed with '@': \"@%systemdir%\\system32\\shell32.dll,-19263\"" << endl
			<< L"RelaunchIcon is of the standard resource form \"%systemdir%\\system32\\shell32.dll,-128\"" << endl
			<< endl
			<< L"Examples:" << endl
			<< L"\tSetAppUserModelId Contoso.GM.ReadMe.1" << endl
			<< L"\tSetAppUserModelId /id Contoso.GM.ReadMe.1 /cmd \"C:\\windows\\notepad.exe C:\\windows\\System32\\drivers\\gmreadme.txt\" /name \"GM Readme\" /icon \"%SystemRoot%\\system32\\shell32.dll,-42\"" << endl
			;
	}
}